'use strict';

var _require = require('../tokenTypes'),
    tokens = _require.tokens;

var NONE = tokens.NONE,
    SPACE = tokens.SPACE,
    COLOR = tokens.COLOR,
    LENGTH = tokens.LENGTH;


module.exports = function (tokenStream) {
  var offsetX = void 0;
  var offsetY = void 0;
  var blurRadius = void 0;
  var color = void 0;

  if (tokenStream.matches(NONE)) {
    tokenStream.expectEmpty();
    return {
      $merge: {
        shadowOffset: { width: 0, height: 0 },
        shadowRadius: 0,
        shadowColor: 'black',
        shadowOpacity: 1
      }
    };
  }

  var didParseFirst = false;
  while (tokenStream.hasTokens()) {
    if (didParseFirst) tokenStream.expect(SPACE);

    if (offsetX === undefined && tokenStream.matches(LENGTH)) {
      offsetX = tokenStream.lastValue;
      tokenStream.expect(SPACE);
      offsetY = tokenStream.expect(LENGTH);

      tokenStream.saveRewindPoint();
      if (tokenStream.matches(SPACE) && tokenStream.matches(LENGTH)) {
        blurRadius = tokenStream.lastValue;
      } else {
        tokenStream.rewind();
      }
    } else if (color === undefined && tokenStream.matches(COLOR)) {
      color = tokenStream.lastValue;
    } else {
      tokenStream.throw();
    }

    didParseFirst = true;
  }

  if (offsetX === undefined) tokenStream.throw();

  var $merge = {
    shadowOffset: { width: offsetX, height: offsetY },
    shadowRadius: blurRadius !== undefined ? blurRadius : 0,
    shadowColor: color !== undefined ? color : 'black',
    shadowOpacity: 1
  };
  return { $merge: $merge };
};